@@ -1,7 +1,8 @@
 from uuid import uuid4
 import json
-import pandas as pd
+from typing import List, Optional
 
+import pandas as pd
 import dash
 from dash.dependencies import Input, Output
 from dash.exceptions import PreventUpdate
@@ -112,6 +113,11 @@ that reads from  `tornadoplot.click_id` if `allow_click` has been specified at i
         """The id of the dcc.Store component that holds click data"""
         return self.ids("click-store")
 
+    @property
+    def high_low_storage_id(self):
+        """The id of the dcc.Store component that holds click data"""
+        return self.ids("high-low-storage")
+
     @staticmethod
     def set_grid_layout(columns):
         return {
@@ -183,6 +189,7 @@ that reads from  `tornadoplot.click_id` if `allow_click` has been specified at i
                         ),
                         dcc.Store(id=self.ids("storage")),
                         dcc.Store(id=self.ids("click-store")),
+                        dcc.Store(id=self.ids("high-low-storage")),
                     ],
                 )
             ]
@@ -190,7 +197,10 @@ that reads from  `tornadoplot.click_id` if `allow_click` has been specified at i
 
     def set_callbacks(self, app):
         @app.callback(
-            Output(self.ids("tornado-graph"), "figure"),
+            [
+                Output(self.ids("tornado-graph"), "figure"),
+                Output(self.ids("high-low-storage"), "data"),
+            ],
             [
                 Input(self.ids("reference"), "value"),
                 Input(self.ids("scale"), "value"),
@@ -222,7 +232,7 @@ that reads from  `tornadoplot.click_id` if `allow_click` has been specified at i
                     locked_si_prefix=data.get("locked_si_prefix", None),
                 )
             except KeyError:
-                return {}
+                return {}, {}
 
         if self.allow_click:
 
@@ -342,7 +352,7 @@ def tornado_plot(
                         "senscase": sens_case,
                         "values": values,
                         "values_ref": scale_to_ref(values, ref_avg, scale),
-                        "reals": list(sens_case_df["REAL"]),
+                        "reals": list(map(int, sens_case_df["REAL"])),
                     }
                 )
         # If `SENSTYPE` is monte carlo get p10, p90
@@ -355,15 +365,15 @@ def tornado_plot(
             p10 = case_df["VALUE"].quantile(0.90)
 
             # Extract list of realizations with values less then reference avg (low)
-            low_reals = list(case_df.loc[case_df["VALUE"] <= ref_avg]["REAL"])
+            low_reals = list(map(int, case_df.loc[case_df["VALUE"] <= ref_avg]["REAL"]))
 
             # Extract list of realizations with values higher then reference avg (high)
-            high_reals = list(case_df.loc[case_df["VALUE"] > ref_avg]["REAL"])
+            high_reals = list(map(int, case_df.loc[case_df["VALUE"] > ref_avg]["REAL"]))
 
             arr.append(
                 {
                     "sensname": sens_name,
-                    "senscase": "p90",
+                    "senscase": "P90",
                     "values": p90,
                     "values_ref": scale_to_ref(p90, ref_avg, scale),
                     "reals": low_reals,
@@ -372,7 +382,7 @@ def tornado_plot(
             arr.append(
                 {
                     "sensname": sens_name,
-                    "senscase": "p10",
+                    "senscase": "P10",
                     "values": p10,
                     "values_ref": scale_to_ref(p10, ref_avg, scale),
                     "reals": high_reals,
@@ -414,6 +424,14 @@ def tornado_plot(
 
     df = sort_by_max(df)
 
+    store_low_high = {
+        sensname: {
+            "real_low": sens_name_df["low_reals"].tolist()[0],
+            "real_high": sens_name_df["high_reals"].tolist()[0],
+        }
+        for sensname, sens_name_df in df.groupby(["sensname"])
+    }
+
     # If percentage, unit is %
     unit_x = "%" if scale == "Percentage" else unit
     # Return tornado data as Plotly figure
@@ -430,7 +448,7 @@ def tornado_plot(
                 f"<br>Case: {label}<br>True Value: "
                 f"{si_prefixed(val, number_format, unit, spaced, locked_si_prefix)}"
                 f"<br>Realizations: "
-                f"{int(min(reals)) if reals else None}-{int(max(reals)) if reals else None}"
+                f"{printable_int_list(reals)}"
                 for x, label, val, reals in zip(
                     df["low"], df["low_label"], df["true_low"], df["low_reals"]
                 )
@@ -450,7 +468,7 @@ def tornado_plot(
                 f"<br>Case: {label}<br>True Value: "
                 f"{si_prefixed(val, number_format, unit, spaced, locked_si_prefix)}"
                 f"<br>Realizations: "
-                f"{int(min(reals)) if reals else None}-{int(max(reals)) if reals else None}"
+                f"{printable_int_list(reals)}"
                 for x, label, val, reals in zip(
                     df["high"], df["high_label"], df["true_high"], df["high_reals"]
                 )
@@ -503,7 +521,10 @@ def tornado_plot(
             ],
         }
     )
-    return {"data": plot_data, "layout": layout}
+    return (
+        {"data": plot_data, "layout": layout},
+        store_low_high,
+    )
 
 
 def calc_low_base(low, high):
@@ -574,3 +595,31 @@ def calc_low_x(low, high):
         base = min(0, high)
         return low - base
     return 0
+
+
+def printable_int_list(integer_list: Optional[List[int]]):
+    """Creates a string out of a list of integers.
+    The string gives a range x-y if all the integers between and
+    including x and y are in the list, otherwise separated by commas.
+    Example: the list [0, 1, 2, 4, 5, 8, 10] becomes '0-2, 4-5, 8, 10'
+    If the input is `None` or the list is empty, the string 'None' is returned.
+    """
+    if not integer_list:
+        return "None"
+
+    sorted_list = sorted(integer_list)
+    prev_number = sorted_list[0]
+    string = str(prev_number)
+
+    for next_number in sorted_list[1:]:
+        if next_number > prev_number + 1:
+            string += (
+                f"{prev_number}" if string.endswith("-") else ""
+            ) + f", {next_number}"
+        elif not string.endswith("-"):
+            string += "-"
+        prev_number = next_number
+
+    if not string.endswith(f", {prev_number}") and string != str(prev_number):
+        string += str(prev_number)
+    return string
