@@ -45,6 +45,10 @@ from ._well_cross_section_fmu import WellCrossSectionFMU
 from ._parameter_parallel_coordinates import ParameterParallelCoordinates
 from ._running_time_analysis_fmu import RunningTimeAnalysisFMU
 from ._relative_permeability import RelativePermeability
+from ._reservoir_simulation_timeseries_regional import (
+    ReservoirSimulationTimeSeriesRegional,
+)
+from ._rft_plotter.rft_plotter import RftPlotter
 
 
 __all__ = [
@@ -66,4 +70,6 @@ __all__ = [
     "ParameterParallelCoordinates",
     "RunningTimeAnalysisFMU",
     "RelativePermeability",
+    "ReservoirSimulationTimeSeriesRegional",
+    "RftPlotter",
 ]
