@@ -3,7 +3,7 @@ from pathlib import Path
 
 import numpy as np
 import pandas as pd
-import dash_table
+from dash_table import DataTable
 import dash_html_components as html
 import dash_core_components as dcc
 from dash.dependencies import Input, Output
@@ -14,7 +14,7 @@ from webviz_config import WebvizPluginABC
 
 from .._datainput.inplace_volumes import extract_volumes
 from .._abbreviations.volume_terminology import volume_description, volume_unit
-from .._abbreviations.number_formatting import TABLE_STATISTICS_BASE
+from .._abbreviations.number_formatting import table_statistics_base
 
 
 class InplaceVolumes(WebvizPluginABC):
@@ -59,7 +59,7 @@ but the following responses are given more descriptive names automatically:
 
 """
 
-    TABLE_STATISTICS = [("Response", {}), ("Group", {})] + TABLE_STATISTICS_BASE
+    TABLE_STATISTICS = [("Group", {})] + table_statistics_base()
 
     def __init__(
         self,
@@ -230,13 +230,10 @@ but the following responses are given more descriptive names automatically:
         dropdowns = []
         for selector in self.selectors:
             elements = list(self.volumes[selector].unique())
-            multi = True
-
             if selector in ["ENSEMBLE", "SOURCE"]:
                 value = elements[0]
             else:
                 value = elements
-
             dropdowns.append(
                 html.Div(
                     children=[
@@ -244,14 +241,14 @@ but the following responses are given more descriptive names automatically:
                             open=True,
                             children=[
                                 html.Summary(selector.lower().capitalize()),
-                                dcc.Dropdown(
+                                wcc.Select(
                                     id=self.selectors_id[selector],
                                     options=[
                                         {"label": i, "value": i} for i in elements
                                     ],
                                     value=value,
-                                    multi=multi,
-                                    clearable=False,
+                                    multi=True,
+                                    size=min(20, len(elements)),
                                 ),
                             ],
                         )
@@ -326,8 +323,9 @@ but the following responses are given more descriptive names automatically:
                     id=self.ids("layout"),
                     children=[
                         html.Div(
+                            style={"flex": 1},
                             children=[
-                                html.P("Filters:", style={"font-weight": "bold"}),
+                                html.Span("Filters:", style={"font-weight": "bold"}),
                                 html.Div(
                                     id=self.ids("filters"),
                                     children=self.selector_dropdowns,
@@ -335,22 +333,33 @@ but the following responses are given more descriptive names automatically:
                             ],
                         ),
                         html.Div(
+                            style={"flex": 5},
                             children=[
                                 self.plot_options_layout,
                                 html.Div(
                                     style={"height": 400},
                                     children=wcc.Graph(id=self.ids("graph")),
                                 ),
+                                html.Div(
+                                    children=[
+                                        html.Div(
+                                            id=self.ids("table_title"),
+                                            style={"textAlign": "center"},
+                                            children="",
+                                        ),
+                                        DataTable(
+                                            id=self.ids("table"),
+                                            sort_action="native",
+                                            filter_action="native",
+                                            page_action="native",
+                                            page_size=10,
+                                        ),
+                                    ],
+                                ),
                             ],
                         ),
                     ],
                 ),
-                dash_table.DataTable(
-                    id=self.ids("table"),
-                    columns=[
-                        {"name": i, "id": i} for i in InplaceVolumes.TABLE_STATISTICS
-                    ],
-                ),
             ]
         )
 
@@ -360,6 +369,7 @@ but the following responses are given more descriptive names automatically:
                 Output(self.ids("graph"), "figure"),
                 Output(self.ids("table"), "data"),
                 Output(self.ids("table"), "columns"),
+                Output(self.ids("table_title"), "children"),
             ],
             self.vol_callback_inputs,
         )
@@ -398,8 +408,11 @@ but the following responses are given more descriptive names automatically:
                         plot_traces.append(trace)
                         table.append(plot_table(dframe, response, name))
             # Column specification
-            columns = table_columns(response)
-            # Else make a graph object
+            columns = [
+                {**{"name": i[0], "id": i[0]}, **i[1]}
+                for i in InplaceVolumes.TABLE_STATISTICS
+            ]
+            # Make a graph object and return
             return (
                 {
                     "data": plot_traces,
@@ -407,12 +420,14 @@ but the following responses are given more descriptive names automatically:
                 },
                 table,
                 columns,
+                f"{volume_description(response)} [{volume_unit(response)}]",
             )
 
         @app.callback(
             [
                 Output(self.selectors_id["ENSEMBLE"], "multi"),
                 Output(self.selectors_id["ENSEMBLE"], "value"),
+                Output(self.selectors_id["ENSEMBLE"], "size"),
             ],
             [Input(self.ids("group"), "value")],
         )
@@ -420,10 +435,11 @@ but the following responses are given more descriptive names automatically:
             """If iteration is selected as group by set the iteration
             selector to allow multiple selections, else use single selection
             """
+            selectors = list(self.volumes["ENSEMBLE"].unique())
             if group_by == "ENSEMBLE":
-                return True, list(self.volumes["ENSEMBLE"].unique())
+                return True, selectors, len(selectors)
 
-            return False, list(self.volumes["ENSEMBLE"].unique())[0]
+            return False, selectors[0], 1
 
         if "SOURCE" in self.selectors:
 
@@ -431,6 +447,7 @@ but the following responses are given more descriptive names automatically:
                 [
                     Output(self.selectors_id["SOURCE"], "multi"),
                     Output(self.selectors_id["SOURCE"], "value"),
+                    Output(self.selectors_id["SOURCE"], "size"),
                 ],
                 [Input(self.ids("group"), "value")],
             )
@@ -438,11 +455,11 @@ but the following responses are given more descriptive names automatically:
                 """If iteration is selected as group by set the iteration
                 selector to allow multiple selections, else use single selection
                 """
-
+                selectors = list(self.volumes["SOURCE"].unique())
                 if group_by == "SOURCE" and "SOURCE" in self.selectors:
-                    return True, list(self.volumes["SOURCE"].unique())
+                    return True, selectors, len(selectors)
 
-                return False, list(self.volumes["SOURCE"].unique())[0]
+                return False, selectors[0], 1
 
 
 @CACHE.memoize(timeout=CACHE.TIMEOUT)
@@ -468,7 +485,6 @@ def plot_table(dframe, response, name):
     values = dframe[response]
     try:
         output = {
-            "Response": volume_description(response),
             "Group": str(name),
             "Minimum": values.min(),
             "Maximum": values.max(),
@@ -483,19 +499,6 @@ def plot_table(dframe, response, name):
     return output
 
 
-@CACHE.memoize(timeout=CACHE.TIMEOUT)
-def table_columns(response):
-    columns = [
-        {**{"name": i[0], "id": i[0]}, **i[1]} for i in InplaceVolumes.TABLE_STATISTICS
-    ]
-    for col in columns:
-        try:
-            col["format"]["locale"]["symbol"] = ["", f"{volume_unit(response)}"]
-        except KeyError:
-            pass
-    return columns
-
-
 @CACHE.memoize(timeout=CACHE.TIMEOUT)
 def plot_layout(plot_type, response, theme):
     layout = {}
@@ -507,17 +510,27 @@ def plot_layout(plot_type, response, theme):
                 "barmode": "overlay",
                 "bargap": 0.01,
                 "bargroupgap": 0.2,
-                "xaxis": {"title": volume_description(response)},
+                "xaxis": {
+                    "title": f"{volume_description(response)} [{volume_unit(response)}]"
+                },
                 "yaxis": {"title": "Count"},
             }
         )
     elif plot_type == "Box plot":
-        layout.update({"yaxis": {"title": volume_description(response)}})
+        layout.update(
+            {
+                "yaxis": {
+                    "title": f"{volume_description(response)} [{volume_unit(response)}]"
+                }
+            }
+        )
     else:
         layout.update(
             {
-                "margin": {"l": 40, "r": 40, "b": 30, "t": 10},
-                "yaxis": {"title": volume_description(response)},
+                "margin": {"l": 60, "r": 40, "b": 30, "t": 10},
+                "yaxis": {
+                    "title": f"{volume_description(response)} [{volume_unit(response)}]"
+                },
                 "xaxis": {"title": "Realization"},
             }
         )
